#include "ets_sys.h"
#include "xap.h"

char *XAP_FILTER_ANY = "XAP_FILTER_ANY";
char *XAP_FILTER_ABSENT = "XAP_FILTER_ABSENT";

/// Return the type of xAP packet
int xapGetType(xAPFrame *frame) {
	if (frame->parsedMsgCount==0) return XAP_MSG_NONE;
	if (strcasecmp(frame->parsedMsg[0].section,"xap-hbeat")==0) return XAP_MSG_HBEAT;
	if (strcasecmp(frame->parsedMsg[0].section,"xap-header")==0) return XAP_MSG_ORDINARY;
	return XAP_MSG_UNKNOWN;
}

/** Fetch the value for a section/key combination.
* NULL will be returned if the section/key is not found
* otherwise a pointer to the value.
*/
char *xapGetValue(xAPFrame *frame, char *section, char *key) {
	int i;	
	for(i=0; i < frame->parsedMsgCount; i++) {
		if (strcasecmp(section, frame->parsedMsg[i].section) == 0 && 
			(
			   (key == NULL && frame->parsedMsg[i].key == NULL) ||
			   (key && frame->parsedMsg[i].key && strcasecmp(key, frame->parsedMsg[i].key) == 0)
			)
		    ) {
			   char *val = frame->parsedMsg[i].value;
			   if(val == NULL && key == NULL) { // Return something for an empty section.
				   return (char *)XAP_FILTER_ANY;
			   }
			   return val;
		   }
	}
	return (char *)NULL;
}

/** Test if a section/key is equal to a value.
* Returns 1 is a match is found otherwise 0
*/
int xapIsValue(xAPFrame *frame, char *section, char *key, char *value) {
	char *kvalue = xapGetValue(frame,section, key);
	return kvalue && strcasecmp(kvalue, value) == 0;
}

/// Right trim control chars and whitespace
static void rtrim(unsigned char *msg,  unsigned char *p) {
	while(*p < 32 && p > msg)
		*p-- = '\0';
}

/** Populates the struct frame.parsedMsg with pointers into dataPacket for the parsing rules.
* dataPacket is modified.
*/
int parseMsg(xAPFrame *frame, unsigned char *dataPacket, int len) {
	enum {
		START_SECTION_NAME, IN_SECTION_NAME, START_KEYNAME, IN_KEYNAME, START_KEYVALUE, IN_KEYVALUE  
	} state = START_SECTION_NAME;
	char *current_section = NULL;
	unsigned char *buf;
	unsigned int i = 0;
	unsigned int keycount = 0;
	frame->parsedMsgCount = 0;
  
	for(buf = dataPacket; i < len && buf < dataPacket+XAP_DATA_LEN; i++, buf++) {
		switch (state) {
		case START_SECTION_NAME:
			if ( (*buf>32) && (*buf<128) ) {
				state = IN_SECTION_NAME;
				current_section = (char *)buf;
			}
			break;
		case IN_SECTION_NAME:
			if (*buf == '{') {
				*buf = '\0';
				rtrim(dataPacket, buf);
				state = START_KEYNAME;
				keycount = 0;
			}
			break;
		case START_KEYNAME:
			if (*buf == '}') {
				// Handle the case of an empty SECTION
				if(keycount == 0) {
					frame->parsedMsg[frame->parsedMsgCount].section = current_section;
					frame->parsedMsg[frame->parsedMsgCount].key = NULL;
					frame->parsedMsg[frame->parsedMsgCount].value = NULL;
					frame->parsedMsgCount++;
				}
				state = START_SECTION_NAME;
			} 
			else if ((*buf>32) && (*buf<128)) {
				frame->parsedMsg[frame->parsedMsgCount].section = current_section;
				frame->parsedMsg[frame->parsedMsgCount].key = (char *)buf;
				state = IN_KEYNAME;
			}
			break;
		case IN_KEYNAME:
			if ((*buf < 32) || (*buf == '=')) {
				*buf = '\0';
				rtrim(dataPacket, buf);
				state = START_KEYVALUE;
			}
			break;
		case START_KEYVALUE:
			if ((*buf>32) && (*buf<128)) {
				state = IN_KEYVALUE;
				frame->parsedMsg[frame->parsedMsgCount].value = (char *)buf;
			}
			break;
		case IN_KEYVALUE:
			if (*buf < 32) {
				*buf = '\0';
				rtrim(dataPacket, buf);
				state = START_KEYNAME;
				keycount++;
				frame->parsedMsgCount++;
				if (frame->parsedMsgCount >= XAP_MSG_ELEMENTS) {
					i = len; // Stop parsing now.
				}
			}
			break;
		}
	}
	return frame->parsedMsgCount;
}
